@@ -1,7 +1,7 @@
 /*
  * Automated Testing Framework (atf)
  *
- * Copyright (c) 2008 The NetBSD Foundation, Inc.
+ * Copyright (c) 2008, 2010 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,11 +44,14 @@
 struct atf_map_citer {
     const struct atf_map *m_map;
     const void *m_entry;
+    atf_list_citer_t m_listiter;
 };
 typedef struct atf_map_citer atf_map_citer_t;
 
 /* Getters. */
+const char *atf_map_citer_key(const atf_map_citer_t);
 const void *atf_map_citer_data(const atf_map_citer_t);
+atf_map_citer_t atf_map_citer_next(const atf_map_citer_t);
 
 /* Operators. */
 bool atf_equal_map_citer_map_citer(const atf_map_citer_t,
@@ -61,11 +64,14 @@ bool atf_equal_map_citer_map_citer(const atf_map_citer_t,
 struct atf_map_iter {
     struct atf_map *m_map;
     void *m_entry;
+    atf_list_iter_t m_listiter;
 };
 typedef struct atf_map_iter atf_map_iter_t;
 
 /* Getters. */
+const char *atf_map_iter_key(const atf_map_iter_t);
 void *atf_map_iter_data(const atf_map_iter_t);
+atf_map_iter_t atf_map_iter_next(const atf_map_iter_t);
 
 /* Operators. */
 bool atf_equal_map_iter_map_iter(const atf_map_iter_t,
@@ -89,6 +95,8 @@ atf_error_t atf_map_init(atf_map_t *);
 void atf_map_fini(atf_map_t *);
 
 /* Getters. */
+atf_map_iter_t atf_map_begin(atf_map_t *);
+atf_map_citer_t atf_map_begin_c(const atf_map_t *);
 atf_map_iter_t atf_map_end(atf_map_t *);
 atf_map_citer_t atf_map_end_c(const atf_map_t *);
 atf_map_iter_t atf_map_find(atf_map_t *, const char *);
@@ -98,4 +106,14 @@ size_t atf_map_size(const atf_map_t *);
 /* Modifiers. */
 atf_error_t atf_map_insert(atf_map_t *, const char *, void *, bool);
 
+/* Macros. */
+#define atf_map_for_each(iter, map) \
+    for (iter = atf_map_begin(map); \
+         !atf_equal_map_iter_map_iter((iter), atf_map_end(map)); \
+         iter = atf_map_iter_next(iter))
+#define atf_map_for_each_c(iter, map) \
+    for (iter = atf_map_begin_c(map); \
+         !atf_equal_map_citer_map_citer((iter), atf_map_end_c(map)); \
+         iter = atf_map_citer_next(iter))
+
 #endif /* ATF_C_MAP_H */
