@@ -1,7 +1,7 @@
 /*
  * Automated Testing Framework (atf)
  *
- * Copyright (c) 2008 The NetBSD Foundation, Inc.
+ * Copyright (c) 2008, 2009 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,7 @@
 
 #include "atf-c/tcr.h"
 
-#include "h_macros.h"
+#include "h_lib.h"
 
 /* ---------------------------------------------------------------------
  * Auxiliary test cases.
@@ -173,6 +173,12 @@ ATF_TC_BODY(config, tcin)
  * but good tests here could allow us to avoid much of the indirect
  * testing done later on. */
 
+/* ---------------------------------------------------------------------
+ * Tests cases for the header file.
+ * --------------------------------------------------------------------- */
+
+HEADER_TC(include, "atf-c/tc.h", "d_include_tc_h.c");
+
 /* ---------------------------------------------------------------------
  * Main.
  * --------------------------------------------------------------------- */
@@ -188,5 +194,8 @@ ATF_TP_ADD_TCS(tp)
     /* Add the test cases for the free functions. */
     /* TODO */
 
+    /* Add the test cases for the header file. */
+    ATF_TP_ADD_TC(tp, include);
+
     return atf_no_error();
 }
