@@ -1,7 +1,7 @@
 //
 // Automated Testing Framework (atf)
 //
-// Copyright (c) 2007 The NetBSD Foundation, Inc.
+// Copyright (c) 2007, 2008 The NetBSD Foundation, Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -12,13 +12,6 @@
 // 2. Redistributions in binary form must reproduce the above copyright
 //    notice, this list of conditions and the following disclaimer in the
 //    documentation and/or other materials provided with the distribution.
-// 3. All advertising materials mentioning features or use of this
-//    software must display the following acknowledgement:
-//        This product includes software developed by the NetBSD
-//        Foundation, Inc. and its contributors.
-// 4. Neither the name of The NetBSD Foundation nor the names of its
-//    contributors may be used to endorse or promote products derived
-//    from this software without specific prior written permission.
 //
 // THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND
 // CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
@@ -34,13 +27,13 @@
 // IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 
-#include <atf.hpp>
+#include <atf-c++.hpp>
 
 ATF_TEST_CASE(pass);
 
 ATF_TEST_CASE_HEAD(pass)
 {
-    set("descr", "An empty test case that always passes");
+    set_md_var("descr", "An empty test case that always passes");
 }
 
 ATF_TEST_CASE_BODY(pass)
@@ -52,19 +45,19 @@ ATF_TEST_CASE(fail);
 
 ATF_TEST_CASE_HEAD(fail)
 {
-    set("descr", "An empty test case that always fails");
+    set_md_var("descr", "An empty test case that always fails");
 }
 
 ATF_TEST_CASE_BODY(fail)
 {
-    ATF_FAIL("On purpose");
+    atf_tc_fail("On purpose");
 }
 
 ATF_TEST_CASE(skip);
 
 ATF_TEST_CASE_HEAD(skip)
 {
-    set("descr", "An empty test case that is always skipped");
+    set_md_var("descr", "An empty test case that is always skipped");
 }
 
 ATF_TEST_CASE_BODY(skip)
@@ -76,8 +69,8 @@ ATF_TEST_CASE(default);
 
 ATF_TEST_CASE_HEAD(default)
 {
-    set("descr", "A test case that passes without explicitly "
-                 "stating it");
+    set_md_var("descr", "A test case that passes without explicitly "
+               "stating it");
 }
 
 ATF_TEST_CASE_BODY(default)
